@@ -6,8 +6,12 @@ package Mojo::DOM58::_CSS;
 
 use strict;
 use warnings;
+use Carp 'croak';
+use Data::Dumper ();
 
-our $VERSION = '2.000';
+use constant DEBUG => $ENV{MOJO_DOM58_CSS_DEBUG} || 0;
+
+our $VERSION = '3.000';
 
 my $ESCAPE_RE = qr/\\[^0-9a-fA-F]|\\[0-9a-fA-F]{1,6}/;
 my $ATTR_RE   = qr/
@@ -16,7 +20,7 @@ my $ATTR_RE   = qr/
   (?:
     (\W)?=                                              # Operator
     (?:"((?:\\"|[^"])*)"|'((?:\\'|[^'])*)'|([^\]]+?))   # Value
-    (?:\s+(i))?                                         # Case-sensitivity
+    (?:\s+(?:(i|I)|s|S))?                               # Case-sensitivity
   )?
   \]
 /x;
@@ -35,18 +39,20 @@ sub tree {
 
 sub matches {
   my $tree = shift->tree;
-  return $tree->[0] ne 'tag' ? undef : _match(_compile(@_), $tree, $tree);
+  return $tree->[0] ne 'tag' ? undef : _match(_compile(@_), $tree, $tree, _root($tree));
 }
 
 sub select     { _select(0, shift->tree, _compile(@_)) }
 sub select_one { _select(1, shift->tree, _compile(@_)) }
 
-sub _ancestor {
-  my ($selectors, $current, $tree, $one, $pos) = @_;
+sub _absolutize { [map { _is_scoped($_) ? $_ : [[['pc', 'scope']], ' ', @$_] } @{shift()}] }
 
-  while ($current = $current->[3]) {
-    return undef if $current->[0] eq 'root' || $current eq $tree;
-    return 1 if _combinator($selectors, $current, $tree, $pos);
+sub _ancestor {
+  my ($selectors, $current, $tree, $scope, $one, $pos) = @_;
+
+  while ($current ne $scope && $current->[0] ne 'root' && ($current = $current->[3])) {
+    return 1     if _combinator($selectors, $current, $tree, $scope, $pos);
+    return undef if $current eq $scope;
     last if $one;
   }
 
@@ -67,26 +73,26 @@ sub _attr {
 }
 
 sub _combinator {
-  my ($selectors, $current, $tree, $pos) = @_;
+  my ($selectors, $current, $tree, $scope, $pos) = @_;
 
   # Selector
   return undef unless my $c = $selectors->[$pos];
   if (ref $c) {
-    return undef unless _selector($c, $current);
+    return undef unless _selector($c, $current, $tree, $scope);
     return 1 unless $c = $selectors->[++$pos];
   }
 
   # ">" (parent only)
-  return _ancestor($selectors, $current, $tree, 1, ++$pos) if $c eq '>';
+  return _ancestor($selectors, $current, $tree, $scope, 1, ++$pos) if $c eq '>';
 
   # "~" (preceding siblings)
-  return _sibling($selectors, $current, $tree, 0, ++$pos) if $c eq '~';
+  return _sibling($selectors, $current, $tree, $scope, 0, ++$pos) if $c eq '~';
 
   # "+" (immediately preceding siblings)
-  return _sibling($selectors, $current, $tree, 1, ++$pos) if $c eq '+';
+  return _sibling($selectors, $current, $tree, $scope, 1, ++$pos) if $c eq '+';
 
   # " " (ancestor)
-  return _ancestor($selectors, $current, $tree, 0, ++$pos);
+  return _ancestor($selectors, $current, $tree, $scope, 0, ++$pos);
 }
 
 sub _compile {
@@ -103,7 +109,10 @@ sub _compile {
     if ($css =~ /\G\s*,\s*/gc) { push @$group, [] }
 
     # Combinator
-    elsif ($css =~ /\G\s*([ >+~])\s*/gc) { push @$selectors, $1 }
+    elsif ($css =~ /\G\s*([ >+~])\s*/gc) {
+      push @$last, ['pc', 'scope'] unless @$last;
+      push @$selectors, $1;
+    }
 
     # Class or ID
     elsif ($css =~ /\G([.#])((?:$ESCAPE_RE\s|\\.|[^,.#:[ >~+])+)/gco) {
@@ -127,8 +136,8 @@ sub _compile {
     elsif ($css =~ /\G:([\w\-]+)(?:\(((?:\([^)]+\)|[^)])+)\))?/gcs) {
       my ($name, $args) = (lc $1, $2);
 
-      # ":matches" and ":not" (contains more selectors)
-      $args = _compile($args, %ns) if $name eq 'matches' || $name eq 'not';
+      # ":is" and ":not" (contains more selectors)
+      $args = _compile($args, %ns) if $name eq 'has' || $name eq 'is' || $name eq 'not';
 
       # ":nth-*" (with An+B notation)
       $args = _equation($args) if $name =~ /^nth-/;
@@ -150,13 +159,14 @@ sub _compile {
       push @$last, ['tag', $name eq '*' ? undef : _name($name), _unescape($ns)];
     }
 
-    else {last}
+    else { pos $css < length $css ? croak "Unknown CSS selector: $css" : last }
   }
 
+  warn qq{-- CSS Selector ($css)\n@{[_dumper($group)]}} if DEBUG;
   return $group;
 }
 
-sub _empty { $_[0][0] eq 'comment' || $_[0][0] eq 'pi' }
+sub _dumper { Data::Dumper->new([@_])->Indent(1)->Sortkeys(1)->Terse(1)->Useqq(1)->Dump }
 
 sub _equation {
   return [0, 0] unless my $equation = shift;
@@ -176,9 +186,24 @@ sub _equation {
   return [$1 eq '-' ? -1 : !length $1 ? 1 : $1, join('', split(' ', $2 || 0))];
 }
 
+sub _is_scoped {
+  my $selector = shift;
+
+  for my $pc (grep { $_->[0] eq 'pc' } map { ref $_ ? @$_ : () } @$selector) {
+
+    # Selector with ":scope"
+    return 1 if $pc->[1] eq 'scope';
+
+    # Argument of functional pseudo-class with ":scope"
+    return 1 if ($pc->[1] eq 'has' || $pc->[1] eq 'is' || $pc->[1] eq 'not') && grep { _is_scoped($_) } @{$pc->[2]};
+  }
+
+  return undef;
+}
+
 sub _match {
-  my ($group, $current, $tree) = @_;
-  _combinator([reverse @$_], $current, $tree, 0) and return 1 for @$group;
+  my ($group, $current, $tree, $scope) = @_;
+  _combinator([reverse @$_], $current, $tree, $scope, 0) and return 1 for @$group;
   return undef;
 }
 
@@ -200,26 +225,33 @@ sub _namespace {
 }
 
 sub _pc {
-  my ($class, $args, $current) = @_;
+  my ($class, $args, $current, $tree, $scope) = @_;
+
+  # ":scope" (root can only be a :scope)
+  return $current eq $scope if $class eq 'scope';
+  return undef              if $current->[0] eq 'root';
 
   # ":checked"
   return exists $current->[2]{checked} || exists $current->[2]{selected}
     if $class eq 'checked';
 
   # ":not"
-  return !_match($args, $current, $current) if $class eq 'not';
+  return !_match($args, $current, $current, $scope) if $class eq 'not';
 
-  # ":matches"
-  return !!_match($args, $current, $current) if $class eq 'matches';
+  # ":is"
+  return !!_match($args, $current, $current, $scope) if $class eq 'is';
+
+  # ":has"
+  return !!_select(1, $current, $args) if $class eq 'has';
 
   # ":empty"
-  return !grep { !_empty($_) } @$current[4 .. $#$current] if $class eq 'empty';
+  return !grep { !($_->[0] eq 'comment' || $_->[0] eq 'pi') } @$current[4 .. $#$current] if $class eq 'empty';
 
   # ":root"
   return $current->[3] && $current->[3][0] eq 'root' if $class eq 'root';
 
-  # ":link" and ":visited"
-  if ($class eq 'link' || $class eq 'visited') {
+  # ":any-link", ":link" and ":visited"
+  if ($class eq 'any-link' || $class eq 'link' || $class eq 'visited') {
     return undef unless $current->[0] eq 'tag' && exists $current->[2]{href};
     return !!grep { $current->[1] eq $_ } qw(a area link);
   }
@@ -250,8 +282,18 @@ sub _pc {
   return undef;
 }
 
+sub _root {
+  my $tree = shift;
+  $tree = $tree->[3] while $tree->[0] ne 'root';
+  return $tree;
+}
+
 sub _select {
-  my ($one, $tree, $group) = @_;
+  my ($one, $scope, $group) = @_;
+
+  # Scoped selectors require the whole tree to be searched
+  my $tree = $scope;
+  ($group, $tree) = (_absolutize($group), _root($scope)) if grep { _is_scoped($_) } @$group;
 
   my @results;
   my @queue = @$tree[($tree->[0] eq 'root' ? 1 : 4) .. $#$tree];
@@ -259,7 +301,7 @@ sub _select {
     next unless $current->[0] eq 'tag';
 
     unshift @queue, @$current[4 .. $#$current];
-    next unless _match($group, $current, $tree);
+    next unless _match($group, $current, $tree, $scope);
     $one ? return $current : push @results, $current;
   }
 
@@ -267,24 +309,26 @@ sub _select {
 }
 
 sub _selector {
-  my ($selector, $current) = @_;
+  my ($selector, $current, $tree, $scope) = @_;
 
+  # The root might be the scope
+  my $is_tag = $current->[0] eq 'tag';
   for my $s (@$selector) {
     my $type = $s->[0];
 
     # Tag
-    if ($type eq 'tag') {
+    if ($is_tag && $type eq 'tag') {
       return undef if defined $s->[1] && $current->[1] !~ $s->[1];
       return undef if defined $s->[2] && !_namespace($s->[2], $current);
     }
 
     # Attribute
-    elsif ($type eq 'attr') { return undef unless _attr(@$s[1, 2], $current) }
+    elsif ($is_tag && $type eq 'attr') { return undef unless _attr(@$s[1, 2], $current) }
 
     # Pseudo-class
-    elsif ($type eq 'pc') { return undef unless _pc(@$s[1, 2], $current) }
+    elsif ($type eq 'pc') { return undef unless _pc(@$s[1, 2], $current, $tree, $scope) }
 
-    # Invalid selector
+    # No match
     else { return undef }
   }
 
@@ -292,17 +336,17 @@ sub _selector {
 }
 
 sub _sibling {
-  my ($selectors, $current, $tree, $immediate, $pos) = @_;
+  my ($selectors, $current, $tree, $scope, $immediate, $pos) = @_;
 
   my $found;
   for my $sibling (@{_siblings($current)}) {
     return $found if $sibling eq $current;
 
     # "+" (immediately preceding sibling)
-    if ($immediate) { $found = _combinator($selectors, $sibling, $tree, $pos) }
+    if ($immediate) { $found = _combinator($selectors, $sibling, $tree, $scope, $pos) }
 
     # "~" (preceding sibling)
-    else { return 1 if _combinator($selectors, $sibling, $tree, $pos) }
+    else { return 1 if _combinator($selectors, $sibling, $tree, $scope, $pos) }
   }
 
   return undef;
